import dash
from dash.dependencies import Input, Output, Event
import dash_core_components as dcc
import dash_html_components as html
from multiprocessing import Value

app = dash.Dash(__name__)
app.layout = html.Div([
    dcc.RadioItems(
        options=[
            {'label': 'Chapter 1', 'value': 'chapter1'},
            {'label': 'Chapter 2', 'value': 'chapter2'},
            {'label': 'Chapter 3', 'value': 'chapter3'},
            {'label': 'Chapter 4', 'value': 'chapter4'},
            {'label': 'Chapter 5', 'value': 'chapter5'}
        ],
        value='chapter1',
        id='toc'
    ),
    html.Div(id='body')
])
for script in dcc._js_dist:
    app.scripts.append_script(script)

chapters = {
    'chapter1': html.Div([
        html.H1('Chapter 1', id='chapter1-header'),
        dcc.Dropdown(
            options=[{'label': i, 'value': i} for i in ['NYC', 'MTL', 'SF']],
            value='NYC',
            id='chapter1-controls'
        ),
        html.Label(id='chapter1-label'),
        dcc.Graph(id='chapter1-graph')
    ]),
    # Chapter 2 has the some of the same components in the same order
    # as Chapter 1. This means that they won't get remounted
    # unless they set their own keys are differently.
    # Switching back and forth between 1 and 2 implicitly
    # tests how components update when they aren't remounted.
    'chapter2': html.Div([
        html.H1('Chapter 2', id='chapter2-header'),
        dcc.RadioItems(
            options=[{'label': i, 'value': i}
                     for i in ['USA', 'Canada']],
            value='USA',
            id='chapter2-controls'
        ),
        html.Label(id='chapter2-label'),
        dcc.Graph(id='chapter2-graph')
    ]),
    # Chapter 3 has a different layout and so the components
    # should get rewritten
    'chapter3': [html.Div(
        html.Div([
            html.H3('Chapter 3', id='chapter3-header'),
            html.Label(id='chapter3-label'),
            dcc.Graph(id='chapter3-graph'),
            dcc.RadioItems(
                options=[{'label': i, 'value': i}
                         for i in ['Summer', 'Winter']],
                value='Winter',
                id='chapter3-controls'
            )
        ])
    )],

    # Chapter 4 doesn't have an object to recursively
    # traverse
    'chapter4': 'Just a string',

    # Chapter 5 contains elements that are bound with events
    'chapter5': [html.Div([
        html.Button(id='chapter5-button'),
        html.Div(id='chapter5-output')
    ])]
}

call_counts = {
    'body': Value('i', 0),
    'chapter1-graph': Value('i', 0),
    'chapter1-label': Value('i', 0),
    'chapter2-graph': Value('i', 0),
    'chapter2-label': Value('i', 0),
    'chapter3-graph': Value('i', 0),
    'chapter3-label': Value('i', 0),
    'chapter5-output': Value('i', 0)
}

@app.callback(Output('body', 'children'), [Input('toc', 'value')])
def display_chapter(toc_value):
    call_counts['body'].value += 1
    return chapters[toc_value]

app.config.supress_callback_exceptions = True

def generate_graph_callback(counterId):
    def callback(value):
        call_counts[counterId].value += 1
        return {
                'data': [{
                    'x': ['Call Counter'],
                    'y': [call_counts[counterId].value],
                    'type': 'bar'
                }],
                'layout': {'title': value}
            }
    return callback

def generate_label_callback(id):
    def update_label(value):
        call_counts[id].value += 1
        return value
    return update_label

for chapter in ['chapter1', 'chapter2', 'chapter3']:
    app.callback(
        Output('{}-graph'.format(chapter), 'figure'),
        [Input('{}-controls'.format(chapter), 'value')]
    )(generate_graph_callback('{}-graph'.format(chapter)))

    app.callback(
        Output('{}-label'.format(chapter), 'children'),
        [Input('{}-controls'.format(chapter), 'value')]
    )(generate_label_callback('{}-label'.format(chapter)))

chapter5_output_children = 'Button clicked'

@app.callback(Output('chapter5-output', 'children'),
              events=[Event('chapter5-button', 'click')])
def display_output():
    call_counts['chapter5-output'].value += 1
    return chapter5_output_children


if __name__ == '__main__':
    app.run_server(debug=True, processes=4)
